@@ -104,7 +104,6 @@ static const char sccsid[] = "@(#)flow.c	5.00 2000/11/01 xlockmore";
 					"*cycles:      10000 \n" \
 					"*ncolors:     200   \n"
 
-# define reshape_flow 0
 # define flow_handle_event 0
 # include "xlockmore.h"		/* in xscreensaver distribution */
 #else /* STANDALONE */
@@ -1111,7 +1110,7 @@ draw_flow (ModeInfo * mi)
 				   are ok, they help to outline the Poincare'
 				   slice. */
 				for(j = i; j < end; j++) Y(j,b) = Y(i,b);
-				begin = i + 1;
+				/*begin = i + 1;*/
 				break;
 			}
 			
@@ -1201,6 +1200,13 @@ draw_flow (ModeInfo * mi)
 	}
 }
 
+ENTRYPOINT void
+reshape_flow(ModeInfo * mi, int width, int height)
+{
+  init_flow (mi);
+}
+
+
 ENTRYPOINT void
 release_flow (ModeInfo * mi)
 {
