@@ -17,14 +17,15 @@
 
 #include "screenhackI.h"
 
-#ifdef HAVE_COCOA
-# include <OpenGL/gl.h>
-# include <OpenGL/glu.h>
-#else  /* !HAVE_COCOA -- real Xlib */
+#ifndef HAVE_COCOA
 # include <GL/glx.h>
 # include <GL/glu.h>
 #endif /* !HAVE_COCOA */
 
+#ifdef HAVE_JWZGLES
+# include "jwzgles.h"
+#endif /* HAVE_JWZGLES */
+
 #include "involute.h"
 #include "normals.h"
 
@@ -33,8 +34,8 @@
 
 
 /* For debugging: if true then in wireframe, do not abbreviate. */
-static Bool wire_all_p = False+1;
-static Bool show_normals_p = False+1;
+static Bool wire_all_p = False;
+static Bool show_normals_p = False;
 
 
 /* Draws an uncapped tube of the given radius extending from top to bottom,
@@ -248,7 +249,6 @@ draw_spokes (int n, GLfloat thickness, int segments,
 
       rb = orb;
     }
-  glEnd();
   return polys;
 }
 
