@@ -445,6 +445,10 @@ init_noof (ModeInfo *mi)
   int i;
   noof_configuration *bp;
 
+#ifdef HAVE_JWZGLES
+  dbuf_p = 1;
+#endif
+
   if (!bps) {
     bps = (noof_configuration *)
       calloc (MI_NUM_SCREENS(mi), sizeof (noof_configuration));
@@ -459,7 +463,6 @@ init_noof (ModeInfo *mi)
   bp->glx_context = init_GL(mi);
 
   glDrawBuffer(dbuf_p ? GL_BACK : GL_FRONT);
-  glClearColor(0.0, 0.0, 0.0, 1.0);
   glEnable(GL_LINE_SMOOTH);
   glShadeModel(GL_FLAT);
   glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
