@@ -568,6 +568,7 @@ DisplaySuperquadrics(superquadricsstruct * sp)
 
 	SetCull(0, sp);
 
+    glScalef(0.7, 0.7, 0.7);  /* jwz: scale it down a bit */
 	polys = DoneScale(sp);
 
 	glPopMatrix();
@@ -665,7 +666,6 @@ InitSuperquadrics(int wfmode, int snorm, int res, int count, float speed, superq
 	if (snorm)
 		sp->shownorms = 1;
 
-    glClearColor(0.0,0.0,0.0,1.0);
     glClearDepth(1.0);
 
 	if (sp->wireframe) {
