@@ -141,7 +141,7 @@ static void setup_random_colormap (struct state *st, XWindowAttributes *xgwa)
     colors[0].pixel = get_pixel_resource (st->dpy, xgwa->colormap,
                                           "background", "Background");
     
-    make_random_colormap (st->dpy, xgwa->visual, xgwa->colormap,
+    make_random_colormap (xgwa->screen, xgwa->visual, xgwa->colormap,
 			  colors+1, &ncolors, True, True, 0, True);
     if (ncolors < 1)
       {
@@ -243,7 +243,6 @@ static void
 setup_display (struct state *st)
 {
     XWindowAttributes xgwa;
-    Colormap cmap;
 
     int cell_size = get_integer_resource (st->dpy, "size", "Integer");
     int osize, alloc_size, oalloc;
@@ -398,8 +397,6 @@ setup_display (struct state *st)
     st->mindeathspeed *= st->diaglim;
     st->maxdeathspeed *= st->diaglim;
 
-    cmap = xgwa.colormap;
-    
     st->windowWidth = xgwa.width;
     st->windowHeight = xgwa.height;
     
@@ -735,6 +732,9 @@ static const char *petri_defaults [] = {
   "*originalcolors:	false",
   "*memThrottle:        22M",	/* don't malloc more than this much.
                                    Scale the pixels up if necessary. */
+#ifdef USE_IPHONE
+  "*ignoreRotation:     True",
+#endif
     0
 };
 
