@@ -52,6 +52,9 @@ static const char *shadebobs_defaults [] = {
   "*cycles:   10",
   "*ncolors:  64",    /* changing this doesn't work particularly well */
   "*delay:    10000",
+#ifdef USE_IPHONE
+  "*ignoreRotation: True",
+#endif
   0
 };
 
@@ -296,12 +299,13 @@ static void Initialize( struct state *st )
 {
 	XGCValues gcValues;
 	XWindowAttributes XWinAttribs;
-	int iBitsPerPixel;
+	/*int iBitsPerPixel;*/
 
 	/* Create the Image for drawing */
 	XGetWindowAttributes( st->dpy, st->window, &XWinAttribs );
 
-	/* Find the preferred bits-per-pixel. (jwz) */
+#if 0
+  /* Find the preferred bits-per-pixel. (jwz) */
 	{
 		int i, pfvc = 0;
 		XPixmapFormatValues *pfv = XListPixmapFormats( st->dpy, &pfvc );
@@ -314,6 +318,7 @@ static void Initialize( struct state *st )
 		if( pfv )
 			XFree (pfv);
 	}
+#endif
 
 	/*  Create the GC. */
 	st->gc = XCreateGC( st->dpy, st->window, 0, &gcValues );
